@@ -1,7 +1,7 @@
 """
 CutoML - A lightweight automl framework for classification and regression tasks.
 
-Copyright (C) 2021  Omkar Udawant
+Copyright (C) 2022  Omkar Udawant
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -56,9 +56,20 @@ class Classifiers:
     def __init__(self,
                  k_folds=5,
                  n_jobs=2,
-                 verbose=1):
+                 verbose=1,
+                 random_state=0):
         self.models = [
-            GaussianNB(),
+            HalvingRandomSearchCV(
+                estimator=GaussianNB(),
+                param_distributions={
+                    'var_smoothing': np.logspace(0, -9, 10)
+                },
+                cv=k_folds,
+                verbose=verbose,
+                n_jobs=n_jobs,
+                scoring='f1_weighted',
+                random_state=random_state
+            ),
             HalvingRandomSearchCV(
                 estimator=BernoulliNB(),
                 param_distributions={
@@ -68,161 +79,156 @@ class Classifiers:
                 cv=k_folds,
                 verbose=verbose,
                 n_jobs=n_jobs,
-                random_state=0
+                scoring='f1_weighted',
+                random_state=random_state
             ),
             HalvingRandomSearchCV(
                 estimator=MultinomialNB(),
                 param_distributions={
-                    'alpha': [1e-3, 1e-2, 1e-1, 1., 10., ],
+                    'alpha': [1e-3, 1e-2, 1e-1, 1., 10., 20.],
                     'fit_prior': [True, False]
                 },
                 cv=k_folds,
                 verbose=verbose,
                 n_jobs=n_jobs,
-                random_state=0
+                scoring='f1_weighted',
+                random_state=random_state
             ),
             HalvingRandomSearchCV(
                 ExtraTreesClassifier(n_jobs=n_jobs,
-                                     random_state=0),
+                                     random_state=random_state),
                 param_distributions={
-                    'n_estimators': [100, 300],
-                    'max_depth': [8, 15, None],
-                    'max_features': ['log2', 'sqrt'],
+                    'n_estimators': [100],
+                    'criterion': ["gini", "entropy"],
+                    'max_features': np.arange(0.05, 1.00, 0.05),
                     'min_samples_split': [2, 5, 10, 15, 20],
                     'min_samples_leaf': [1, 2, 5, 10],
+                    'bootstrap': [True, False]
                 },
                 cv=k_folds,
-                random_state=0,
+                random_state=random_state,
                 verbose=verbose,
+                scoring='f1_weighted',
                 n_jobs=n_jobs
             ),
             HalvingRandomSearchCV(
                 KNeighborsClassifier(n_jobs=n_jobs),
                 param_distributions={
-                    'n_neighbors': [2, 4, 8, 10],
+                    'n_neighbors': [2, 4, 8, 10, 12],
+                    'weights': ["uniform", "distance"],
                     'p': [1, 2]
                 },
                 cv=k_folds,
-                random_state=0,
+                random_state=random_state,
                 verbose=verbose,
+                scoring='f1_weighted',
                 n_jobs=n_jobs
             ),
             HalvingRandomSearchCV(
                 LogisticRegression(n_jobs=n_jobs,
-                                   random_state=0),
+                                   random_state=random_state),
                 param_distributions={
                     'penalty': ["l1", "l2"],
                     'C': np.logspace(0, 3, 4),
-                    'max_iter': np.arange(start=100, stop=1001, step=500)
+                    'max_iter': [100, 500]
                 },
                 cv=k_folds,
-                random_state=0,
-                verbose=verbose,
-                n_jobs=n_jobs
-            ),
-            HalvingRandomSearchCV(
-                LogisticRegression(n_jobs=n_jobs,
-                                   random_state=0),
-                param_distributions={
-                    'penalty': ["l2"],
-                    'solver': ['newton-cg', 'saga', 'sag'],
-                    'C': np.logspace(0, 3, 4),
-                    'max_iter': np.arange(start=100, stop=1001, step=500)
-                },
-                cv=k_folds,
-                random_state=0,
+                random_state=random_state,
                 verbose=verbose,
+                scoring='f1_weighted',
                 n_jobs=n_jobs
             ),
             HalvingRandomSearchCV(
                 SGDClassifier(n_jobs=n_jobs,
-                              random_state=0),
+                              random_state=random_state),
                 param_distributions={
                     'loss': ['log', 'hinge', 'modified_huber', 'squared_hinge',
                              'perceptron'],
                     'penalty': ['elasticnet'],
                     'alpha': [0.0, 0.01, 0.001],
                     'learning_rate': ['invscaling', 'constant'],
-                    'l1_ratio': [0.25, 0.0, 1.0, 0.75, 0.5],
-                    'eta0': [0.1, 1.0, 0.01],
-                    'power_t': [0.5, 0.0, 1.0, 0.1, 10.0, 50.0]
                 },
                 cv=k_folds,
-                random_state=0,
+                random_state=random_state,
                 verbose=verbose,
+                scoring='f1_weighted',
                 n_jobs=n_jobs
             ),
             HalvingRandomSearchCV(
-                LinearSVC(random_state=0),
+                LinearSVC(random_state=random_state),
                 param_distributions={
                     'penalty': ["l2"],
                     'dual': [True],
-                    'max_iter': np.arange(start=100, stop=1001, step=500),
+                    'max_iter': [100, 500],
                     'tol': [1e-3, 1e-2, 1e-1],
                     'C': np.logspace(0, 3, 4),
                 },
-                random_state=0,
+                random_state=random_state,
                 verbose=verbose,
+                scoring='f1_weighted',
                 cv=k_folds,
                 n_jobs=n_jobs
             ),
             HalvingRandomSearchCV(
-                LinearSVC(random_state=0),
+                LinearSVC(random_state=random_state),
                 param_distributions={
                     'penalty': ["l1", "l2"],
-                    'max_iter': np.arange(start=100, stop=1001, step=500),
+                    'max_iter': [100, 500],
                     'dual': [False],
                     'tol': [1e-3, 1e-2, 1e-1],
                     'C': np.logspace(0, 3, 4)
                 },
-                random_state=0,
+                random_state=random_state,
                 verbose=verbose,
+                scoring='f1_weighted',
                 cv=k_folds,
                 n_jobs=n_jobs
             ),
             HalvingRandomSearchCV(
                 RandomForestClassifier(
                     n_jobs=n_jobs,
-                    random_state=0),
+                    random_state=random_state),
                 param_distributions={
-                    'n_estimators': [100, 300],
+                    'n_estimators': [100],
                     'max_features': ['log2', 'sqrt'],
                     'min_samples_split': [2, 5, 10, 15, 20],
                     'min_samples_leaf': [1, 2, 5, 10],
                 },
                 cv=k_folds,
-                random_state=0,
+                random_state=random_state,
                 verbose=verbose,
+                scoring='f1_weighted',
                 n_jobs=n_jobs
             ),
             HalvingRandomSearchCV(
-                GradientBoostingClassifier(random_state=0),
+                GradientBoostingClassifier(random_state=random_state),
                 param_distributions={
-                    'n_estimators': [100, 300],
-                    'learning_rate': [1e-3, 1e-2, 1e-1, 0.5, 1.],
+                    'n_estimators': [100],
+                    'learning_rate': [1e-3, 1e-2, 1e-1, 1.],
                 },
                 cv=k_folds,
-                random_state=0,
+                random_state=random_state,
                 verbose=verbose,
+                scoring='f1_weighted',
                 n_jobs=n_jobs
             ),
             HalvingRandomSearchCV(
                 XGBClassifier(n_jobs=n_jobs,
-                              random_state=0),
+                              random_state=random_state),
                 param_distributions={
-                    'n_estimators': [100, 300],
-                    'learning_rate': [1e-3, 1e-2, 1e-1, 0.5, 1.],
+                    'n_estimators': [100],
+                    'learning_rate': [1e-3, 1e-2, 1e-1, 1.],
                     'min_child_weight': range(1, 21),
                     'subsample': np.arange(0.05, 1.0, 0.05),
-                    'min_child_weight': range(1, 21),
                 },
                 cv=k_folds,
-                random_state=0,
+                random_state=random_state,
                 verbose=verbose,
+                scoring='f1_weighted',
                 n_jobs=n_jobs
             ),
             HalvingRandomSearchCV(
-                DecisionTreeClassifier(random_state=0),
+                DecisionTreeClassifier(random_state=random_state),
                 param_distributions={
                     'criterion': ["gini", "entropy"],
                     'max_depth': range(1, 21),
@@ -230,14 +236,15 @@ class Classifiers:
                     'min_samples_leaf': range(1, 21)
                 },
                 cv=k_folds,
-                random_state=0,
+                random_state=random_state,
                 verbose=verbose,
+                scoring='f1_weighted',
                 n_jobs=n_jobs
             ),
             HalvingRandomSearchCV(
-                MLPClassifier(random_state=0),
+                MLPClassifier(random_state=random_state),
                 param_distributions={
-                    'max_iter': np.arange(start=100, stop=1001, step=500),
+                    'max_iter': [100, 500],
                     'hidden_layer_sizes': [
                         (10, 30, 10),
                         (50, 50, 50),
@@ -251,9 +258,10 @@ class Classifiers:
                     'learning_rate': ['constant', 'adaptive'],
                 },
                 cv=k_folds,
-                random_state=0,
+                random_state=random_state,
                 verbose=verbose,
                 n_jobs=n_jobs,
+                scoring='f1_weighted',
             )
         ]
 
@@ -262,11 +270,12 @@ class Regressors:
     def __init__(self,
                  k_folds=5,
                  n_jobs=2,
-                 verbose=1):
+                 verbose=1,
+                 random_state=0):
         self.models = [
             LinearRegression(n_jobs=n_jobs),
             HalvingRandomSearchCV(
-                estimator=ElasticNet(random_state=0),
+                estimator=ElasticNet(random_state=random_state),
                 param_distributions={
                     'l1_ratio': np.arange(0.0, 1.0, 0.05),
                     'tol': [1e-3, 1e-2, 1e-1]
@@ -274,29 +283,31 @@ class Regressors:
                 cv=k_folds,
                 verbose=verbose,
                 n_jobs=n_jobs,
-                random_state=0
+                scoring='r2',
+                random_state=random_state
             ),
             HalvingRandomSearchCV(
-                estimator=ExtraTreesRegressor(random_state=0),
+                estimator=ExtraTreesRegressor(random_state=random_state),
                 param_distributions={
-                    'n_estimators': [100, 300],
-                    'max_depth': [8, 15, None],
-                    'max_features': ['log2', 'sqrt'],
+                    'n_estimators': [100],
+                    'criterion': ["gini", "entropy"],
+                    'max_features': np.arange(0.05, 1.00, 0.05),
                     'min_samples_split': [2, 5, 10, 15, 20],
                     'min_samples_leaf': [1, 2, 5, 10],
+                    'bootstrap': [True, False]
                 },
                 cv=k_folds,
                 verbose=verbose,
                 n_jobs=n_jobs,
-                random_state=0
+                scoring='r2',
+                random_state=random_state
             ),
             HalvingRandomSearchCV(
-                estimator=GradientBoostingRegressor(random_state=0),
+                estimator=GradientBoostingRegressor(random_state=random_state),
                 param_distributions={
-                    'n_estimators': [100, 300],
+                    'n_estimators': [100],
                     'loss': ["ls", "lad", "huber", "quantile"],
-                    'learning_rate': [1e-3, 1e-2, 1e-1, 0.5, 1.],
-                    'max_depth': [8, 15, None],
+                    'learning_rate': [1e-3, 1e-2, 1e-1, 1.],
                     'max_features': ['log2', 'sqrt'],
                     'min_samples_split': [2, 5, 10, 15, 20],
                     'min_samples_leaf': [1, 2, 5, 10],
@@ -305,75 +316,82 @@ class Regressors:
                 cv=k_folds,
                 verbose=verbose,
                 n_jobs=n_jobs,
-                random_state=0
+                scoring='r2',
+                random_state=random_state
             ),
             HalvingRandomSearchCV(
-                estimator=AdaBoostRegressor(random_state=0),
+                estimator=AdaBoostRegressor(random_state=random_state),
                 param_distributions={
-                    'n_estimators': [100, 300],
-                    'learning_rate': [1e-3, 1e-2, 1e-1, 0.5, 1.],
+                    'n_estimators': [100],
+                    'learning_rate': [1e-3, 1e-2, 1e-1, 1.],
                     'loss': ["linear", "square", "exponential"]
                 },
                 cv=k_folds,
                 verbose=verbose,
                 n_jobs=n_jobs,
-                random_state=0
+                scoring='r2',
+                random_state=random_state
             ),
             HalvingRandomSearchCV(
-                estimator=DecisionTreeRegressor(random_state=0),
+                estimator=DecisionTreeRegressor(random_state=random_state),
                 param_distributions={
                     'max_depth': range(1, 11),
-                    'min_samples_split': range(2, 21),
-                    'min_samples_leaf': range(1, 21)
+                    'min_samples_split': [2, 5, 10, 15, 20],
+                    'min_samples_leaf': [1, 2, 5, 10],
                 },
                 cv=k_folds,
                 verbose=verbose,
                 n_jobs=n_jobs,
-                random_state=0
+                scoring='r2',
+                random_state=random_state
             ),
             HalvingRandomSearchCV(
                 estimator=KNeighborsRegressor(n_jobs=n_jobs),
                 param_distributions={
-                    'n_neighbors': [2, 4, 8, 10],
+                    'n_neighbors': [2, 4, 8, 10, 12],
                     'p': [1, 2]
                 },
                 cv=k_folds,
                 verbose=verbose,
                 n_jobs=n_jobs,
-                random_state=0
+                scoring='r2',
+                random_state=random_state
             ),
             HalvingRandomSearchCV(
-                estimator=Ridge(random_state=0),
+                estimator=Ridge(random_state=random_state),
                 param_distributions={
                     'alpha': [0.001, 0.01, 0.1, 1, 10, 100]
                 },
                 cv=k_folds,
                 verbose=verbose,
                 n_jobs=n_jobs,
-                random_state=0
+                scoring='r2',
+                random_state=random_state
             ),
             HalvingRandomSearchCV(
-                estimator=Lasso(random_state=0),
+                estimator=Lasso(random_state=random_state),
                 param_distributions={
                     'alpha': [0.001, 0.01, 0.1, 1, 10, 100]
                 },
                 cv=k_folds,
                 verbose=verbose,
                 n_jobs=n_jobs,
-                random_state=0
+                scoring='r2',
+                random_state=random_state
             ),
             HalvingRandomSearchCV(
-                estimator=LassoLars(random_state=0),
+                estimator=LassoLars(random_state=random_state),
                 param_distributions={
                     'alpha': [0.001, 0.01, 0.1, 1, 10, 100]
                 },
                 cv=k_folds,
                 verbose=verbose,
                 n_jobs=n_jobs,
-                random_state=0
+                scoring='r2',
+                random_state=random_state
             ),
             HalvingRandomSearchCV(
-                estimator=LinearSVR(random_state=0),
+                estimator=LinearSVR(random_state=random_state),
                 param_distributions={
                     'loss': ["epsilon_insensitive",
                              "squared_epsilon_insensitive"],
@@ -385,10 +403,11 @@ class Regressors:
                 cv=k_folds,
                 verbose=verbose,
                 n_jobs=n_jobs,
-                random_state=0
+                scoring='r2',
+                random_state=random_state
             ),
             HalvingRandomSearchCV(
-                estimator=LinearSVR(random_state=0),
+                estimator=LinearSVR(random_state=random_state),
                 param_distributions={
                     'loss': ["squared_epsilon_insensitive"],
                     'dual': [False],
@@ -399,14 +418,15 @@ class Regressors:
                 cv=k_folds,
                 verbose=verbose,
                 n_jobs=n_jobs,
-                random_state=0
+                scoring='r2',
+                random_state=random_state
             ),
             HalvingRandomSearchCV(
                 estimator=RandomForestRegressor(
                     n_jobs=n_jobs,
-                    random_state=0),
+                    random_state=random_state),
                 param_distributions={
-                    'n_estimators': [100, 300],
+                    'max_iter': [100],
                     'max_features': ['log2', 'sqrt', ],
                     'min_samples_split': [2, 5, 10, 15, 20],
                     'min_samples_leaf': [1, 2, 5, 10],
@@ -414,15 +434,16 @@ class Regressors:
                 cv=k_folds,
                 verbose=verbose,
                 n_jobs=n_jobs,
-                random_state=0
+                scoring='r2',
+                random_state=random_state
             ),
             HalvingRandomSearchCV(
                 estimator=XGBRegressor(
                     n_jobs=n_jobs,
-                    random_state=0),
+                    random_state=random_state),
                 param_distributions={
-                    'n_estimators': [100, 300],
-                    'learning_rate': [1e-3, 1e-2, 1e-1, 0.5, 1.],
+                    'n_estimators': [100],
+                    'learning_rate': [1e-3, 1e-2, 1e-1, 1.],
                     'min_child_weight': range(1, 21),
                     'subsample': np.arange(0.05, 1.0, 0.05),
                     'min_child_weight': range(1, 21),
@@ -430,22 +451,21 @@ class Regressors:
                 cv=k_folds,
                 verbose=verbose,
                 n_jobs=n_jobs,
-                random_state=0
+                scoring='r2',
+                random_state=random_state
             ),
             HalvingRandomSearchCV(
-                estimator=SGDRegressor(random_state=0),
+                estimator=SGDRegressor(random_state=random_state),
                 param_distributions={
                     'loss': ['squared_loss', 'huber', 'epsilon_insensitive'],
                     'penalty': ['elasticnet'],
                     'alpha': [0.0, 0.01, 0.001],
                     'learning_rate': ['invscaling', 'constant'],
-                    'l1_ratio': [0.25, 0.0, 1.0, 0.75, 0.5],
-                    'eta0': [0.1, 1.0, 0.01],
-                    'power_t': [0.5, 0.0, 1.0, 0.1, 10.0, 50.0]
                 },
                 cv=k_folds,
                 verbose=verbose,
                 n_jobs=n_jobs,
-                random_state=0
+                scoring='r2',
+                random_state=random_state
             )
         ]
